@@ -1,9 +1,8 @@
 module Language.Haskell.GhcMod.Doc where
 
-import GHC (DynFlags, GhcMonad)
-import qualified GHC as G
+import GHC
 import Language.Haskell.GhcMod.Gap (withStyle, showDocWith)
-import Outputable (SDoc, PprStyle, mkUserStyle, Depth(AllTheWay), neverQualify)
+import Outputable
 import Pretty (Mode(..))
 
 showPage :: DynFlags -> PprStyle -> SDoc -> String
@@ -12,9 +11,14 @@ showPage dflag style = showDocWith dflag PageMode . withStyle dflag style
 showOneLine :: DynFlags -> PprStyle -> SDoc -> String
 showOneLine dflag style = showDocWith dflag OneLineMode . withStyle dflag style
 
+-- showForUser :: DynFlags -> PrintUnqualified -> SDoc -> String
+-- showForUser dflags unqual sdoc =
+--     showDocWith dflags PageMode $
+--       runSDoc sdoc $ initSDocContext dflags $ mkUserStyle unqual AllTheWay
+
 getStyle :: GhcMonad m => m PprStyle
 getStyle = do
-    unqual <- G.getPrintUnqual
+    unqual <- getPrintUnqual
     return $ mkUserStyle unqual AllTheWay
 
 styleUnqualified :: PprStyle
