@@ -9,19 +9,18 @@ module Language.Haskell.GhcMod.FillSig (
 
 import Data.Char (isSymbol)
 import Data.Function (on)
+import Data.Functor
 import Data.List (find, nub, sortBy)
 import qualified Data.Map as M
-import Data.Maybe (isJust, catMaybes)
+import Data.Maybe (catMaybes)
+import Text.PrettyPrint (($$), text, nest)
+import Prelude
+
 import Exception (ghandle, SomeException(..))
 import GHC (GhcMonad, Id, ParsedModule(..), TypecheckedModule(..), DynFlags,
             SrcSpan, Type, GenLocated(L))
 import qualified GHC as G
 import qualified Name as G
-import qualified Language.Haskell.GhcMod.Gap as Gap
-import Language.Haskell.GhcMod.Convert
-import Language.Haskell.GhcMod.Monad
-import Language.Haskell.GhcMod.SrcUtils
-import Language.Haskell.GhcMod.Types
 import Outputable (PprStyle)
 import qualified Type as Ty
 import qualified HsBinds as Ty
@@ -31,6 +30,21 @@ import qualified HsPat as Ty
 import qualified Language.Haskell.Exts.Annotated as HE
 import Djinn.GHC
 
+import qualified Language.Haskell.GhcMod.Gap as Gap
+import Language.Haskell.GhcMod.Convert
+import Language.Haskell.GhcMod.DynFlags
+import Language.Haskell.GhcMod.Monad
+import Language.Haskell.GhcMod.SrcUtils
+import Language.Haskell.GhcMod.Logging (gmLog)
+import Language.Haskell.GhcMod.Pretty (showDoc)
+import Language.Haskell.GhcMod.Doc
+import Language.Haskell.GhcMod.Types
+import Language.Haskell.GhcMod.FileMapping (fileModSummaryWithMapping)
+
+#if __GLASGOW_HASKELL__ >= 710
+import GHC (unLoc)
+#endif
+
 ----------------------------------------------------------------
 -- INTIAL CODE FROM FUNCTION OR INSTANCE SIGNATURE
 ----------------------------------------------------------------
@@ -62,26 +76,31 @@ sig :: IOish m
     -> Int          -- ^ Line number.
     -> Int          -- ^ Column number.
     -> GhcModT m String
-sig file lineNo colNo = ghandle handler body
-  where
-    body = inModuleContext file $ \dflag style -> do
-        opt <- options
-        modSum <- Gap.fileModSummary file
-        whenFound opt (getSignature modSum lineNo colNo) $ \s -> case s of
+sig file lineNo colNo =
+    runGmlT' [Left file] deferErrors $ ghandle fallback $ do
+      oopts <- outputOpts
+      style <- getStyle
+      dflag <- G.getSessionDynFlags
+      modSum <- fileModSummaryWithMapping file
+      whenFound oopts (getSignature modSum lineNo colNo) $ \s ->
+        case s of
           Signature loc names ty ->
-            ("function", fourInts loc, map (initialBody dflag style ty) names)
+              ("function", fourInts loc, map (initialBody dflag style ty) names)
+
           InstanceDecl loc cls ->
-             ("instance", fourInts loc, map (\x -> initialBody dflag style (G.idType x) x)
-                                            (Ty.classMethods cls))
+            let body x = initialBody dflag style (G.idType x) x
+            in ("instance", fourInts loc, body `map` Ty.classMethods cls)
+
           TyFamDecl loc name flavour vars ->
             let (rTy, initial) = initialTyFamString flavour
-             in (rTy, fourInts loc, [initial ++ initialFamBody dflag style name vars])
-
-    handler (SomeException _) = do
-      opt <- options
+                body = initialFamBody dflag style name vars
+            in (rTy, fourInts loc, [initial ++ body])
+  where
+    fallback (SomeException _) = do
+      oopts <- outputOpts
       -- Code cannot be parsed by ghc module
       -- Fallback: try to get information via haskell-src-exts
-      whenFound opt (getSignatureFromHE file lineNo colNo) $ \x -> case x of
+      whenFound oopts (getSignatureFromHE file lineNo colNo) $ \x -> case x of
         HESignature loc names ty ->
           ("function", fourIntsHE loc, map (initialBody undefined undefined ty) names)
         HEFamSignature loc flavour name vars ->
@@ -97,7 +116,11 @@ getSignature modSum lineNo colNo = do
     p@ParsedModule{pm_parsed_source = ps} <- G.parseModule modSum
     -- Inspect the parse tree to find the signature
     case listifyParsedSpans ps (lineNo, colNo) :: [G.LHsDecl G.RdrName] of
+#if __GLASGOW_HASKELL__ >= 710
+      [L loc (G.SigD (Ty.TypeSig names (L _ ty) _))] ->
+#else
       [L loc (G.SigD (Ty.TypeSig names (L _ ty)))] ->
+#endif
         -- We found a type signature
         return $ Just $ Signature loc (map G.unLoc names) ty
       [L _ (G.InstD _)] -> do
@@ -125,7 +148,12 @@ getSignature modSum lineNo colNo = do
                         G.TypeFamily -> Open
                         G.DataFamily -> Data
 #endif
-#if __GLASGOW_HASKELL__ >= 706
+
+#if __GLASGOW_HASKELL__ >= 710
+            getTyFamVarName x = case x of
+                L _ (G.UserTyVar n)     -> n
+                L _ (G.KindedTyVar (G.L _ n) _) -> n
+#elif __GLASGOW_HASKELL__ >= 706
             getTyFamVarName x = case x of
                 L _ (G.UserTyVar n)     -> n
                 L _ (G.KindedTyVar n _) -> n
@@ -144,7 +172,8 @@ getSignature modSum lineNo colNo = do
                       return $ InstanceDecl loc cls
 
 -- Get signature from haskell-src-exts
-getSignatureFromHE :: GhcMonad m => FilePath -> Int -> Int -> m (Maybe HESigInfo)
+getSignatureFromHE :: (MonadIO m, GhcMonad m) =>
+    FilePath -> Int -> Int -> m (Maybe HESigInfo)
 getSignatureFromHE file lineNo colNo = do
   presult <- liftIO $ HE.parseFile file
   return $ case presult of
@@ -220,9 +249,11 @@ initialHead1 :: String -> [FnArg] -> [String] -> String
 initialHead1 fname args elts =
   case initialBodyArgs1 args elts of
     []      -> fname
-    arglist -> if isSymbolName fname
-               then head arglist ++ " " ++ fname ++ " " ++ unwords (tail arglist)
-               else fname ++ " " ++ unwords arglist
+    arglist
+      | isSymbolName fname ->
+        head arglist ++ " " ++ fname ++ " " ++ unwords (tail arglist)
+      | otherwise ->
+        fname ++ " " ++ unwords arglist
 
 initialBodyArgs1 :: [FnArg] -> [String] -> [String]
 initialBodyArgs1 args elts = take (length args) elts
@@ -238,12 +269,24 @@ class FnArgsInfo ty name | ty -> name, name -> ty where
 
 instance FnArgsInfo (G.HsType G.RdrName) (G.RdrName) where
   getFnName dflag style name = showOccName dflag style $ Gap.occName name
-  getFnArgs (G.HsForAllTy _ _ _ (L _ iTy))  = getFnArgs iTy
+#if __GLASGOW_HASKELL__ >= 710
+  getFnArgs (G.HsForAllTy _ _ _ _ (L _ iTy))
+#else
+  getFnArgs (G.HsForAllTy _ _ _ (L _ iTy))
+#endif
+    = getFnArgs iTy
+
   getFnArgs (G.HsParTy (L _ iTy))           = getFnArgs iTy
   getFnArgs (G.HsFunTy (L _ lTy) (L _ rTy)) =
       (if fnarg lTy then FnArgFunction else FnArgNormal):getFnArgs rTy
     where fnarg ty = case ty of
-              (G.HsForAllTy _ _ _ (L _ iTy)) -> fnarg iTy
+#if __GLASGOW_HASKELL__ >= 710
+              (G.HsForAllTy _ _ _ _ (L _ iTy)) ->
+#else
+              (G.HsForAllTy _ _ _ (L _ iTy)) ->
+#endif
+                fnarg iTy
+
               (G.HsParTy (L _ iTy))          -> fnarg iTy
               (G.HsFunTy _ _)                -> True
               _                              -> False
@@ -301,48 +344,61 @@ refine :: IOish m
        -> Int          -- ^ Column number.
        -> Expression   -- ^ A Haskell expression.
        -> GhcModT m String
-refine file lineNo colNo expr = ghandle handler body
+refine file lineNo colNo (Expression expr) =
+  ghandle handler $
+    runGmlT' [Left file] deferErrors $ do
+      oopts <- outputOpts
+      style <- getStyle
+      dflag <- G.getSessionDynFlags
+      modSum <- fileModSummaryWithMapping file
+      p <- G.parseModule modSum
+      tcm@TypecheckedModule{tm_typechecked_source = tcs} <- G.typecheckModule p
+      ety <- G.exprType expr
+      whenFound oopts (findVar dflag style tcm tcs lineNo colNo) $
+        \(loc, name, rty, paren) ->
+            let eArgs = getFnArgs ety
+                rArgs = getFnArgs rty
+                diffArgs' = length eArgs - length rArgs
+                diffArgs  = if diffArgs' < 0 then 0 else diffArgs'
+                iArgs = take diffArgs eArgs
+                txt = initialHead1 expr iArgs (infinitePrefixSupply name)
+             in (fourInts loc, doParen paren txt)
   where
-    body = inModuleContext file $ \dflag style -> do
-        opt <- options
-        modSum <- Gap.fileModSummary file
-        p <- G.parseModule modSum
-        tcm@TypecheckedModule{tm_typechecked_source = tcs} <- G.typecheckModule p
-        ety <- G.exprType expr
-        whenFound opt (findVar dflag style tcm tcs lineNo colNo) $
-          \(loc, name, rty, paren) ->
-              let eArgs = getFnArgs ety
-                  rArgs = getFnArgs rty
-                  diffArgs' = length eArgs - length rArgs
-                  diffArgs  = if diffArgs' < 0 then 0 else diffArgs'
-                  iArgs = take diffArgs eArgs
-                  text = initialHead1 expr iArgs (infinitePrefixSupply name)
-               in (fourInts loc, doParen paren text)
-
-    handler (SomeException _) = emptyResult =<< options
+   handler (SomeException ex) = do
+     gmLog GmException "refining" $
+           text "" $$ nest 4 (showDoc ex)
+     emptyResult =<< outputOpts
 
 -- Look for the variable in the specified position
-findVar :: GhcMonad m => DynFlags -> PprStyle
-                      -> G.TypecheckedModule -> G.TypecheckedSource
-                      -> Int -> Int -> m (Maybe (SrcSpan, String, Type, Bool))
+findVar
+  :: GhcMonad m
+  => DynFlags
+  -> PprStyle
+  -> G.TypecheckedModule
+  -> G.TypecheckedSource
+  -> Int
+  -> Int
+  -> m (Maybe (SrcSpan, String, Type, Bool))
 findVar dflag style tcm tcs lineNo colNo =
-  let lst = sortBy (cmp `on` G.getLoc) $
-                listifySpans tcs (lineNo, colNo) :: [G.LHsExpr Id]
-  in case lst of
-      e@(L _ (G.HsVar i)):others ->
-        do tyInfo <- Gap.getType tcm e
-           let name = getFnName dflag style i
-           if (name == "undefined" || head name == '_') && isJust tyInfo
-              then let Just (s,t) = tyInfo
-                       b = case others of  -- If inside an App, we need
-                                           -- parenthesis
-                             [] -> False
-                             L _ (G.HsApp (L _ a1) (L _ a2)):_ ->
-                               isSearchedVar i a1 || isSearchedVar i a2
-                             _  -> False
-                    in return $ Just (s, name, t, b)
-              else return Nothing
-      _ -> return Nothing
+  case lst of
+    e@(L _ (G.HsVar i)):others -> do
+      tyInfo <- Gap.getType tcm e
+      case tyInfo of
+        Just (s, typ)
+          | name == "undefined" || head name == '_' ->
+            return $ Just (s, name, typ, b)
+          where
+            name = getFnName dflag style i
+            -- If inside an App, we need parenthesis
+            b = case others of
+                  L _ (G.HsApp (L _ a1) (L _ a2)):_ ->
+                    isSearchedVar i a1 || isSearchedVar i a2
+                  _  -> False
+        _ -> return Nothing
+    _ -> return Nothing
+  where
+    lst :: [G.LHsExpr Id]
+    lst = sortBy (cmp `on` G.getLoc) $ listifySpans tcs (lineNo, colNo)
 
 infinitePrefixSupply :: String -> [String]
 infinitePrefixSupply "undefined" = repeat "undefined"
@@ -366,17 +422,18 @@ auto :: IOish m
      -> Int          -- ^ Line number.
      -> Int          -- ^ Column number.
      -> GhcModT m String
-auto file lineNo colNo = ghandle handler body
-  where
-    body = inModuleContext file $ \dflag style -> do
-        opt <- options
-        modSum <- Gap.fileModSummary file
+auto file lineNo colNo =
+  ghandle handler $ runGmlT' [Left file] deferErrors $ do
+        oopts <- outputOpts
+        style <- getStyle
+        dflag <- G.getSessionDynFlags
+        modSum <- fileModSummaryWithMapping file
         p <- G.parseModule modSum
         tcm@TypecheckedModule {
                  tm_typechecked_source = tcs
                , tm_checked_module_info = minfo
                } <- G.typecheckModule p
-        whenFound' opt (findVar dflag style tcm tcs lineNo colNo) $ \(loc, _name, rty, paren) -> do
+        whenFound' oopts (findVar dflag style tcm tcs lineNo colNo) $ \(loc, _name, rty, paren) -> do
           topLevel <- getEverythingInTopLevel minfo
           let (f,pats) = getPatsForVariable tcs (lineNo,colNo)
               -- Remove self function to prevent recursion, and id to trim
@@ -395,8 +452,11 @@ auto file lineNo colNo = ghandle handler body
           djinns <- djinn True (Just minfo) env rty (Max 10) 100000
           return ( fourInts loc
                  , map (doParen paren) $ nub (djinnsEmpty ++ djinns))
-
-    handler (SomeException _) = emptyResult =<< options
+ where
+   handler (SomeException ex) = do
+     gmLog GmException "auto-refining" $
+           text "" $$ nest 4 (showDoc ex)
+     emptyResult =<< outputOpts
 
 -- Functions we do not want in completions
 notWantedFuns :: [String]
@@ -443,7 +503,11 @@ getPatsForVariable tcs (lineNo, colNo) =
 #else
                     :: [G.LMatch Id]
 #endif
+#if __GLASGOW_HASKELL__ >= 710
+              (L _ (G.Match _ pats _ _):_) = m
+#else
               (L _ (G.Match pats _ _):_) = m
+#endif
            in (funId, pats)
         _ -> (error "This should never happen", [])
 
@@ -478,7 +542,13 @@ getBindingsForRecPat (Ty.PrefixCon args) =
 getBindingsForRecPat (Ty.InfixCon (L _ a1) (L _ a2)) =
     M.union (getBindingsForPat a1) (getBindingsForPat a2)
 getBindingsForRecPat (Ty.RecCon (Ty.HsRecFields { Ty.rec_flds = fields })) =
-    getBindingsForRecFields fields
- where getBindingsForRecFields [] = M.empty
-       getBindingsForRecFields (Ty.HsRecField {Ty.hsRecFieldArg = (L _ a)}:fs) =
-         M.union (getBindingsForPat a) (getBindingsForRecFields fs)
+    getBindingsForRecFields (map unLoc' fields)
+ where
+#if __GLASGOW_HASKELL__ >= 710
+   unLoc' = unLoc
+#else
+   unLoc' = id
+#endif
+   getBindingsForRecFields [] = M.empty
+   getBindingsForRecFields (Ty.HsRecField {Ty.hsRecFieldArg = (L _ a)}:fs) =
+       M.union (getBindingsForPat a) (getBindingsForRecFields fs)
