@@ -101,7 +101,7 @@ unloaded modules are loaded")
 (defun ghc-boot (n)
   (prog2
       (message "Initializing...")
-      (ghc-sync-process "boot\n" n)
+      (ghc-sync-process "boot\n" n nil 'skip-map-file)
     (message "Initializing...done")))
 
 (defun ghc-load-modules (mods)
@@ -127,7 +127,7 @@ unloaded modules are loaded")
   (interactive)
   (if (ghc-should-scroll)
       (ghc-scroll-completion-buffer)
-      (ghc-try-complete)))
+    (ghc-try-complete)))
 
 (defun ghc-should-scroll ()
   (let ((window (ghc-completion-window)))
@@ -265,7 +265,7 @@ unloaded modules are loaded")
   (let (ret)
     (save-excursion
       (goto-char (point-min))
-      (while (re-search-forward "^import\\( *qualified\\)? +\\([^\n ]+\\)" nil t)
+      (while (re-search-forward "^import +\\(qualified\\)? *\\([^\n ]+\\)" nil t)
 	(ghc-add ret (match-string-no-properties 2))
 	(forward-line)))
     ret))
