@@ -3,6 +3,7 @@ module BrowseSpec where
 import Control.Applicative
 import Language.Haskell.GhcMod
 import Test.Hspec
+import Prelude
 
 import TestUtils
 import Dir
@@ -16,17 +17,18 @@ spec = do
 
     describe "browse -d Data.Either" $ do
         it "contains functions (e.g. `either') including their type signature" $ do
-            syms <- run defaultOptions { detailed = True }
+            syms <- run defaultOptions { optDetailed = True }
                     $ lines <$> browse "Data.Either"
             syms `shouldContain` ["either :: (a -> c) -> (b -> c) -> Either a b -> c"]
 
         it "contains type constructors (e.g. `Left') including their type signature" $ do
-            syms <- run defaultOptions { detailed = True}
+            syms <- run defaultOptions { optDetailed = True}
                     $ lines <$> browse "Data.Either"
             syms `shouldContain` ["Left :: a -> Either a b"]
 
     describe "`browse' in a project directory" $ do
-        it "lists symbols defined in a a local module (e.g. `Baz.baz)" $ do
-            withDirectory_ "test/data" $ do
-                syms <- runID $ lines <$> browse "Baz"
-                syms `shouldContain` ["baz"]
+        it "can list symbols defined in a a local module" $ do
+            withDirectory_ "test/data/ghc-mod-check/" $ do
+                syms <- runD $ lines <$> browse "Data.Foo"
+                syms `shouldContain` ["foo"]
+                syms `shouldContain` ["fibonacci"]
